@@ -23,8 +23,10 @@
 #ifndef DLG_BOOKMARKS_H
 #define DLG_BOOKMARKS_H
 
-#include "window.h"
+#include "gth-browser.h"
+
+void dlg_edit_bookmarks        (GthBrowser *browser);
+void dlg_edit_bookmarks_update (GtkWidget  *dialog);
 
-void dlg_edit_bookmarks (GThumbWindow *window);
 
 #endif /* DLG_BOOKMARKS_H */
